'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var shell = _interopDefault(require('shelljs'));
var child_process = require('child_process');
var os = _interopDefault(require('os'));
var fs = _interopDefault(require('fs-extra'));
var path = _interopDefault(require('path'));
var set = _interopDefault(require('lodash.set'));
var get = _interopDefault(require('lodash.get'));

/* eslint-env node */
/* eslint-disable no-console */
function usage() {
  console.log(`Initialize workflow-home directory

Usage:
  npm init workflow-home [target]
  npx create-workflow-home [target]
  yarn create workflow-home [target]

Initializes the current directory or target if given
with the create-workflow-home initializer.
`);
}

function parse(args) {
  if (args.length < 1) {
    console.error('Invalid number of arguments');
    console.log();
    usage();
    process.exit(1);
  }

  const [node, cmd, targetFolder] = args; // eslint-disable-line

  if (args.length === 2) {
    return {};
  } else if (args.length === 3) {
    return { targetFolder };
  } else {
    console.error("Invalid arguments: '" + args.join(' ') + "'");
    console.log();
    usage();
    process.exit(1);
  }
}

/* eslint-disable import/prefer-default-export */

function which(cmd) {
  return shell.exec(`which ${cmd}`, { silent: true, async: false });
}

/* eslint-env node */

const platform = process.platform;

function isRunningI3() {
  try {
    child_process.execSync("i3-msg 'exec echo 1'");
    return true;
  } catch (e) {
    return false;
  }
}

function isRunningWmctrl() {
  return which('wmctrl');
}

const wm = (() => {
  switch (platform) {
    case 'win32':
    case 'darwin':
      return 'default';
    case 'linux':
      {
        if (isRunningI3()) {
          return 'i3';
        } else if (isRunningWmctrl()) {
          return 'wmctrl';
        }
        return 'unknown';
      }
  }
  return 'unknown';
})();

const dev = process.env.WORKFLOW_DEV_MODE === 'true';
const homedir = process.env.WORKFLOW_HOME;
const devhomedir = process.env.WORKFLOW_DEV_HOME;

const baseFolder = dev ? devhomedir || `${__dirname}/../../workflow-template` : homedir || `${os.homedir()}/.workflow`;

/* eslint-env node */

const { targetFolder } = parse(process.argv);

Promise.resolve().then(ensureTargetExists).then(createPackageJson).then(createConfigJs).then(copyStaticFiles).then(() => console.log('Initialized:', resolveTargetFolder())).catch(err => console.error(err)); // eslint-disable-line no-console

function ensureTargetExists() {
  fs.mkdirpSync(path.resolve(resolveTargetFolder()));
}

function createPackageJson() {
  const pkgPath = resolveTarget('package.json');

  if (fs.existsSync(pkgPath)) {
    throw new Error('Upgrade existing folder is not supported');
  }

  const versions = readVersions();
  const pkgTemplatePath = resolveSource('templates', 'package.json');

  renderTemplate(pkgTemplatePath, pkgPath, versions);
}

function createConfigJs() {
  const configPath = resolveTarget('config.js');
  const configTemplatePath = resolveSource('templates', 'config.js');
  const versions = readVersions();

  renderTemplate(configTemplatePath, configPath, versions);
}

function copyStaticFiles() {
  const source = resolveSource('static/');
  const destination = resolveTarget();

  fs.copySync(source, destination);
}

function readVersions() {
  if (!readVersions.versions) {
    const versionPath = resolveSource('versions.json');

    const versionsString = fs.readFileSync(versionPath, 'utf8');
    const versions = JSON.parse(versionsString);

    readVersions.versions = set(versions, 'workflow.wm', get(versions, `workflow.wm.${platform}.${wm}`));
  }

  return readVersions.versions;
}

function renderTemplate(template, path$$1, values) {
  const templateString = fs.readFileSync(template, 'utf8');

  const renderedTemplate = templateString.replace(/{{([a-z.]*)}}/g, (_, prelude) => get(values, prelude));

  fs.writeFileSync(path$$1, renderedTemplate);
}

function resolveSource(...args) {
  return path.join(__dirname, '..', 'files', ...args);
}

function resolveTarget(...args) {
  if (args) {
    return path.join(resolveTargetFolder(), ...args);
  } else {
    return resolveTargetFolder();
  }
}

function resolveTargetFolder() {
  if (targetFolder) {
    return path.resolve(targetFolder);
  } else {
    return path.resolve();
  }
}
